@@ -21,12 +21,13 @@ module Annex (
 	setField,
 	getFlag,
 	getField,
+	addCleanup,
 	gitRepo,
 	inRepo,
 	fromRepo,
 ) where
 
-import Control.Monad.State
+import Control.Monad.State.Strict
 import Control.Monad.Trans.Control (StM, MonadBaseControl, liftBaseWith, restoreM)
 import Control.Monad.Base (liftBase, MonadBase)
 import System.Posix.Types (Fd)
@@ -35,12 +36,14 @@ import Common
 import qualified Git
 import qualified Git.Config
 import Git.CatFile
+import Git.CheckAttr
 import qualified Git.Queue
 import Types.Backend
 import qualified Types.Remote
 import Types.Crypto
 import Types.BranchState
 import Types.TrustLevel
+import Utility.State
 import qualified Utility.Matcher
 import qualified Data.Map as M
 
@@ -74,13 +77,14 @@ data AnnexState = AnnexState
 	{ repo :: Git.Repo
 	, backends :: [BackendA Annex]
 	, remotes :: [Types.Remote.RemoteA Annex]
-	, repoqueue :: Git.Queue.Queue
 	, output :: OutputType
 	, force :: Bool
 	, fast :: Bool
 	, auto :: Bool
 	, branchstate :: BranchState
+	, repoqueue :: Maybe Git.Queue.Queue
 	, catfilehandle :: Maybe CatFileHandle
+	, checkattrhandle :: Maybe CheckAttrHandle
 	, forcebackend :: Maybe String
 	, forcenumcopies :: Maybe Int
 	, limit :: Matcher (FilePath -> Annex Bool)
@@ -90,6 +94,7 @@ data AnnexState = AnnexState
 	, lockpool :: M.Map FilePath Fd
 	, flags :: M.Map String Bool
 	, fields :: M.Map String String
+	, cleanup :: M.Map String (Annex ())
 	}
 
 newState :: Git.Repo -> AnnexState
@@ -97,13 +102,14 @@ newState gitrepo = AnnexState
 	{ repo = gitrepo
 	, backends = []
 	, remotes = []
-	, repoqueue = Git.Queue.new
 	, output = NormalOutput
 	, force = False
 	, fast = False
 	, auto = False
 	, branchstate = startBranchState
+	, repoqueue = Nothing
 	, catfilehandle = Nothing
+	, checkattrhandle = Nothing
 	, forcebackend = Nothing
 	, forcenumcopies = Nothing
 	, limit = Left []
@@ -113,6 +119,7 @@ newState gitrepo = AnnexState
 	, lockpool = M.empty
 	, flags = M.empty
 	, fields = M.empty
+	, cleanup = M.empty
 	}
 
 {- Create and returns an Annex state object for the specified git repo. -}
@@ -125,27 +132,20 @@ run s a = runStateT (runAnnex a) s
 eval :: AnnexState -> Annex a -> IO a
 eval s a = evalStateT (runAnnex a) s
 
-{- Gets a value from the internal state, selected by the passed value
- - constructor. -}
-getState :: (AnnexState -> a) -> Annex a
-getState = gets
-
-{- Applies a state mutation function to change the internal state. 
- -
- - Example: changeState $ \s -> s { output = QuietOutput }
- -}
-changeState :: (AnnexState -> AnnexState) -> Annex ()
-changeState = modify
-
 {- Sets a flag to True -}
 setFlag :: String -> Annex ()
 setFlag flag = changeState $ \s ->
-	s { flags = M.insert flag True $ flags s }
+	s { flags = M.insertWith' const flag True $ flags s }
 
 {- Sets a field to a value -}
 setField :: String -> String -> Annex ()
 setField field value = changeState $ \s ->
-	s { fields = M.insert field value $ fields s }
+	s { fields = M.insertWith' const field value $ fields s }
+
+{- Adds a cleanup action to perform. -}
+addCleanup :: String -> Annex () -> Annex ()
+addCleanup uid a = changeState $ \s ->
+	s { cleanup = M.insertWith' const uid a $ cleanup s }
 
 {- Checks if a flag was set. -}
 getFlag :: String -> Annex Bool
