@@ -27,6 +27,7 @@ module Annex.Content (
 
 import Control.Exception (bracket_)
 import System.Posix.Types
+import System.IO.Unsafe (unsafeInterleaveIO)
 
 import Common.Annex
 import Logs.Location
@@ -149,16 +150,16 @@ prepTmp key = do
 getViaTmpUnchecked :: Key -> (FilePath -> Annex Bool) -> Annex Bool
 getViaTmpUnchecked key action = do
 	tmp <- prepTmp key
-	success <- action tmp
-	if success
-		then do
+	ifM (action tmp)
+		( do
 			moveAnnex key tmp
 			logStatus key InfoPresent
 			return True
-		else do
+		, do
 			-- the tmp file is left behind, in case caller wants
 			-- to resume its transfer
 			return False
+		)
 
 {- Creates a temp file, runs an action on it, and cleans up the temp file. -}
 withTmp :: Key -> (FilePath -> Annex a) -> Annex a
@@ -229,15 +230,15 @@ moveAnnex :: Key -> FilePath -> Annex ()
 moveAnnex key src = do
 	dest <- inRepo $ gitAnnexLocation key
 	let dir = parentDir dest
-	e <- liftIO $ doesFileExist dest
-	if e
-		then liftIO $ removeFile src
-		else liftIO $ do
+	liftIO $ ifM (doesFileExist dest)
+		( removeFile src
+		, do
 			createDirectoryIfMissing True dir
 			allowWrite dir -- in case the directory already exists
 			moveFile src dest
 			preventWrite dest
 			preventWrite dir
+		)
 
 withObjectLoc :: Key -> ((FilePath, FilePath) -> Annex a) -> Annex a
 withObjectLoc key a = do
@@ -290,19 +291,20 @@ moveBad key = do
 
 {- List of keys whose content exists in .git/annex/objects/ -}
 getKeysPresent :: Annex [Key]
-getKeysPresent = getKeysPresent' =<< fromRepo gitAnnexObjectDir
-getKeysPresent' :: FilePath -> Annex [Key]
-getKeysPresent' dir = do
-	exists <- liftIO $ doesDirectoryExist dir
-	if not exists
-		then return []
-		else liftIO $ do
-			-- 2 levels of hashing
-			levela <- dirContents dir
-			levelb <- mapM dirContents levela
-			contents <- mapM dirContents (concat levelb)
-			let files = concat contents
-			return $ mapMaybe (fileKey . takeFileName) files
+getKeysPresent = liftIO . traverse (2 :: Int) =<< fromRepo gitAnnexObjectDir
+	where
+		traverse depth dir = do
+			contents <- catchDefaultIO (dirContents dir) []
+			if depth == 0
+				then continue (mapMaybe (fileKey . takeFileName) contents) []
+				else do
+					let deeper = traverse (depth - 1)
+					continue [] (map deeper contents)
+		continue keys [] = return keys
+		continue keys (a:as) = do
+			{- Force lazy traversal with unsafeInterleaveIO. -}
+			morekeys <- unsafeInterleaveIO a
+			continue (morekeys++keys) as
 
 {- Things to do to record changes to content when shutting down.
  -
@@ -312,12 +314,12 @@ getKeysPresent' dir = do
 saveState :: Bool -> Annex ()
 saveState oneshot = do
 	Annex.Queue.flush False
-	unless oneshot $ do
-		alwayscommit <- fromMaybe True . Git.configTrue
+	unless oneshot $
+		ifM alwayscommit
+			( Annex.Branch.commit "update" , Annex.Branch.stage)
+	where
+		alwayscommit = fromMaybe True . Git.configTrue
 			<$> fromRepo (Git.Config.get "annex.alwayscommit" "")
-		if alwayscommit
-			then Annex.Branch.commit "update"
-			else Annex.Branch.stage
 
 {- Downloads content from any of a list of urls. -}
 downloadUrl :: [Url.URLString] -> FilePath -> Annex Bool
@@ -336,10 +338,9 @@ preseedTmp key file = go =<< inAnnex key
 			ok <- copy
 			when ok $ liftIO $ allowWrite file
 			return ok
-		copy = do
-			present <- liftIO $ doesFileExist file
-			if present
-				then return True
-				else do
+		copy = ifM (liftIO $ doesFileExist file)
+				( return True
+				, do
 					s <- inRepo $ gitAnnexLocation key
 					liftIO $ copyFileExternal s file
+				)
