@@ -10,6 +10,8 @@ module Backend.URL (
 	fromUrl
 ) where
 
+import Data.Hash.MD5
+
 import Common.Annex
 import Types.Backend
 import Types.Key
@@ -24,5 +26,16 @@ backend = Backend {
 	fsckKey = Nothing
 }
 
-fromUrl :: String -> Key
-fromUrl url = stubKey { keyName = url, keyBackendName = "URL" }
+fromUrl :: String -> Maybe Integer -> Key
+fromUrl url size = stubKey
+	{ keyName = key
+	, keyBackendName = "URL"
+	, keySize = size
+	}
+	where
+		-- when it's not too long, use the url as the key name
+		-- 256 is the absolute filename max, but use a shorter
+		-- length because this is not the entire key filename.
+		key
+			| length url < 128 = url
+			| otherwise = take 128 url ++ "-" ++ md5s (Str url)
